'DS3231 RTC & Temperature Program
'Program is compatible with the older DS1307 except it doesn't have the onboard digital thermometer so ignore these readings!

#terminal 4800							'auto-open terminal display @ 4800 baud (8, N, 1)

'variable declarations
symbol tempC   = w0						'temperature 16-bit word consists of 8-bit registers b0 & b1

symbol tempLSB = b0						'temperature least significant byte (LSB)
symbol tempMSB = b1						'temperature most significant byte (MSB) 

symbol sign    = b2						'temperature +/- indicator

symbol secs    = b3
symbol mins    = b4
symbol hours   = b5
symbol dow     = b6						'dow = day of the week where $01 = Sunday & $07 = Saturday, etc.
symbol day     = b7
symbol month   = b8
symbol year    = b9

symbol AM_PM   = b10

symbol control = b11					'$0 enable SQW (square wave) output ($4 = disable)

symbol D0      = b12					'working registers for BDC conversion & ASCII display
symbol D1      = b13
symbol D2      = b14
symbol D3      = b15
symbol D4      = b16
symbol D5      = b17

'set DS3231 RTC values using 24-hour time (local/UTC) format (adjust as required)
year    = $17
month   = $02
day     = $11
dow     = $07							'dow = day of week where $01 = Sunday & $07 = Saturday (user definable)
hours   = $12
mins    = $35
secs    = $30

'square wave outputs: $0 = 1 Hz, $8 = 1024 Hz, $10 = 4096 Hz & $18 = 8192 Hz, $4 = off

control = $0															

AM_PM = ""								'reset AM_PM flag

sertxd(cr, lf, "Press button to continue...")

'set I2C bus speed to 100 kbps @ PICAXE default clock speed (4 MHz)
hi2csetup i2cmaster, %11010000, i2cslow, i2cbyte	

'push button connected PICAXE 08M2+ C.3 (pin 4) via 10000 ohm pulldown resistor
'when setting RTC, press button 1 second earlier than required time to set to allow for programming time of RTC
do
	
	if pinC.3 = 1 then exit				'wait for logical high (5 VDC) pulse (push button pressed)	
	
loop

'*********************************************************************
'uncomment following line to set DS3231 RTC using preprogrammed values
'*********************************************************************
'hi2cout $0, (secs, mins, hours, dow, day, month, year)

'set required SQW output frequency (required every power up)
hi2cout $0E, (control)

Main:

	'read BCD (binary code decimal) data from DS3231 RTC module 
	hi2cin $0, (secs, mins, hours, dow, day, month, year) 			
	 	
	'uncomment to display time using 12-hour format (default is 24-hour format)
	gosub Format_12_Hour

	'convert BCD to ASCII (text) values & display results to PICAXE terminal display
   	BCDToASCII hours, D5, D4
   
   	if D5 = "0" then					'zero blanking (don't dislay leading 0 in hours)
   	
   		D5 = " "
   	
   	endif
   
   'convert DS3231 RTC BDC values to ASCII so we can display results on serial monitor (or LCD)
   BcdToASCII mins, D3, D2
   BcdToASCII secs, D1, D0   
   
   'display current time in ASCII as hours:mins:secs
   sertxd (cr, lf, D5, D4, ":", D3, D2, ":", D1, D0)

	if AM_PM <> "" then					'apply "AM" or "PM" time tag, if required (set if Format_12_Hour set)
	
  		sertxd (" ", AM_PM, "M  ")
   
   	else
   	
   		sertxd ("  ")
   	
   	endif

   'display current date in ASCII year/month/day format
   BcdToASCII day, D5, D4 
   BcdToASCII month, D3, D2
   BcdToASCII year, D1, D0   
   
   'note default century is assumed to be 21st (not 19th!)
   sertxd ("20", D1, D0, "/", D3, D2, "/", D5, D4, "  ")
                
   'display current day of week (dow)
	select dow 
	
		case 1
    		sertxd("Sun")
	
		case 2
			sertxd("Mon")
   
   		case 3
    		sertxd("Tue")
    	
    	case 4
			sertxd("Wed")
			
      	case 5
      		sertxd("Thu")
    
  		case 6
    		sertxd("Fri")
      
      	case 7
    		sertxd("Sat")

	endselect
	
	'read & display current ambient air temperature in degrees Celsius
	hi2cin $11, (tempMSB, tempLSB) 		'read DS3231 internal thermometer (-40.00 C  to  +85.00 C) 
	
	sign = "+" 							'assume temperature reading is positive (initially)
    
	if tempMSB > 127 then 				'is it negative?
		
		tempC = -tempC					'yes, negate reading & change sign
		
		sign  = "-"
		
	endif

	'need to convert BCD to decimal (Celsius) when working with DS3231 & simliar RTCs
	tempC = tempC ** 25600				'returns high word (negative tempratures not used)

	BinToASCII tempC, D0, D1, D2, D3, D4
    
	if D0 = "0" then					'blank any leading zeros (if required)
		
		D0 = " "
	
	end if

	if D1 = "0" then
	
		D1 = " "

	endif 
            
    'display temperature with 0.25 C resolution
	sertxd("  ", sign, D0, D1, D2, ".", D3, D4, " C")
	
	pause 840
	
goto Main

'display time in 12-hour format (24-hour format is default)
Format_12_Hour:
   
	if hours < $12 then					'morning (A) or afternoon (P) the "M" is added when displaying time

		AM_PM = "A"
		
	else
	
		AM_PM = "P"
		
	endif
	
	'adjust BCD 24-hour time format (if required) to 12-hour format
	if hours = $20 or hours = $21 then
	
		hours = hours - $6
		
	endif
	
   hours = hours // $12
   
   if hours = $0 then
   
   		hours = $12
   	
   endif
   
return
