#REM
Original PICAXE-08M QRSS bcn using AD9850/AD9851 (DDS-30/DDS-60) DDS by Paul Wilton, M1CNK, (c) 2009
Original Morse code routines based on original PICAXE CW bcn program by Steve Paige, VK6HV, (c) 2008

QRSS + WSPR modifications/additions by Robert C. Mazur, VA3ROM (July/September 2016)

History:
	v1 - normal CW only
 	v2 - offers CW, DFCW or FSKCW options
 	v3 - uses LED plus tune switch (modified for other purpose VA3ROM July 2016)
	v4 - modified by VA3ROM with interrupt driven DS3231 RTC, new Morse code, QRSS & WSPR routines
	     (program redesigned for PICAXE 08M2+ & will not run on older PICAXE 08/M series)
    v5 - added SMT (graphical) CW mode VA3ROM (March 2017)
	v6 - added triangle CW mode VA3ROM (April 2017)

QUICK START USE INSTRUCTIONS (It's really, really a good idea to read the program code & associated comments!)
_______________________________________________________________________________________________________________________________

	QRSS 6 mode (6 sec dot length) used for QRSS 6 CW beacons & QRSS 1 mode (1 sec dot length) for SMT graphical CW bcn
	WSPR bcn (if defined) is transmitted first followed by QRSS bcn (you select combination to use)

	1. Select QRSS bcn mode to use (but only one mode can be active at any one time) plus WSPR (active or not)
	2. Enter QRSS & WSPR bcn data (note pertinent remarks in that section)
	3. Enter EEPROM frequencies in reverse order hex bytes (4 per freq & note pertinent remarks in that section)
	4. Change DDS_CTRL as required (AD9850 uses $0 while AD9851 uses $9)
	5. Remember to start bcn tx on even minute (if using WSPR bcn) & ensure WSPR + QRSS bcns or just QRSS bcn tx time
	   fits inside 10 min tx frame (QRSS standard to allow for QRSS frame stacking/averaging in use since mid-2010)		  
	6. The sertxd statements are used for debugging with PICAXE IDE serial monitor & can be left "active" or comment them out

Feel free to make any modifications. The DS3231 RTC isn't used for any date/time functions but it can be, if you want.	  
During long term tests, the DIS3231 TCXO crystal controlled clock only gains 1 to 1.5 secs a month which is good enough
even for WSPR bcn timing over several months.	 

__________________________________________________MORSE CODE CHARACTERS________________________________________________________

    A .-    B -...  C -.-.  D -..    E .      F ..-.
    G --.   H ....  I ..    J .---   K -.-    L .-..
    M --    N -.    O ---   P .--.   Q --.-   R .-.
    S ...   T -     U ..-   V ...-   W .--    X -..-	Y -.--  Z --.. 
    1 .---- 2 ..--- 3 ...-- 4 ....-  5 .....  6 -.... 7 --... 8 ---.. 9 ----.  0 -----

	FULL STOP          [.]  .-.-.-     COLON                   [:]  ---...
    COMMA              [,]  --..--     SEMICOLON               [;]  -.-.-.
    QUESTION MARK      [?]  ..--..     EQUAL SIGN (DOUBLE DASH)[=]  -...-
    APOSTROPHE         [']  .----.     PLUS                    [+]  .-.-.
    EXCLAMATION MARK   [!]  -.-.--     HYPHEN minus            [-]  -....-
    FWD SLASH          [/]  -..-.      UNDERSCORE              [_]  ..--.-
    PARENTHESIS open   [(]  -.--.      QUOTATION MARK          ["]  .-..-.
    PARENTHESIS closed [)]  -.--.-     DOLLAR SIGN             [$]  ...-..-
    AMPERSAND          [&]  .-...      AT SIGN (COMMAT) [WR]   [@]  .--.-.

________________________________________ENCODED MORSE CODE ALPHANUMERIC CHARACTER SET__________________________________________

    A   6,   B  17,   C  21,   D   9,   E   2,    F  20,
    G  11,   H  16,   I   4,   J  30,   K  13,    L  18,
    M   7,   N   5,   O  15,   P  22,   Q  27,    R  10,
    S   8,   T   3,   U  12,   V  24,   W  14,    X  25,	Y  29,   Z  19, 

    1  62,   2  60,   3  56,   4  48,   5   32,  6  33,   7  35,   8  39,   9  47,   0   63

	"CUT" ABBREVIATED NUMBERS ARE USED (BUT NOT ALLOWED IN CALLSIGNS) TO SHORTEN COMMUNICATIONS
	THIS IS ONE VARIATION OF CUT NUMBERS (AMONG SEVERAL) COMMONLY USED IN AMATEUR RADIO

	T = 0, A = 1, U = 2, V = 3, 4 = 4, E = 5, 6 = 6, G = 7, D = 8, N = 9

    FULL STOP          [.]  106     COLON                  [:]  71
    COMMA              [,]  115     SEMICOLON              [;]  85
    QUESTION MARK      [?]  76      EQUAL SIGN(DOUBLE DASH)[=]  49
    APOSTROPHE         [']  94      PLUS                   [+]  42
    EXCLAMATION MARK   [!]  117     HYPHEN minus           [-]  97
    FWD SLASH          [/]  41      UNDERSCORE             [_]  108
    PARENTHESIS open   [(]  45      QUOTATION MARK         ["]  82
	PARENTHESIS closed [)]  109     DOLLAR SIGN            [$]  200
    AMPERSAND          [&]  34      AT SIGN (COMMAT) [WR]  [@]  86	SPACE [ ] 0

__________________________________________ENCODED SMTCW ALPHANUMERIC CHARACTER SET_____________________________________________

	Designed by Robert C. Mazur, VA3ROM, using MEPTController program by Con Wassilieff, ZL2AFP
	250 marks end of SMT CW character so add character spacing (characters by default end in 250)
	251 marks end of SMT CW group so add word spacing change 250 to 251 in QRSS_LOOKUP data table
	255 marks end of SMT CW bcn so stop transmission change 250 as required in QRSS_LOOKUP data table

	0 = 10,7,0,13,4,0,13,4,0,10,7,250
	1 = 6,13,10,7,4,250
	2 = 13,0,11,13,4,9,13,4,13,7,250
	3 = 13,0,13,4,8,13,4,8,11,6,250
	4 = 10,7,10,0,10,13,10,7,4,10,250
	5 = 13,7,4,13,8,4,13,8,11,4,250
	6 = 11,8,13,5,8,13,4,8,11,4,250
	7 = 4,13,0,10,4,7,0,4,250
	8 = 11,6,8,13,4,8,13,4,8,11,6,250
	9 = 13,6,9,4,13,9,4,11,9,6,250

	A = 13,10,7,0,9,5,9,4,9,5,13,10,7,250
	B = 13,10,7,4,13,8,4,13,8,4,11,6,250
	C = 10,7,13,4,0,13,4,0,13,4,250	
	D = 13,10,7,4,13,0,4,13,0,4,11,7,250
	E = 10,7,13,8,4,13,8,4,13,4,250
	F = 13,10,7,4,9,4,9,4,0,4,250
	G = 10,7,13,4,0,13,4,9,13,4,9,11,250
	H = 13,10,7,4,8,0,8,0,8,13,10,7,4,250
	I = 13,0,4,13,10,7,4,13,0,4,250
	J = 10,12,0,13,0,13,0,12,10,7,4,250
	K = 13,10,7,4,8,0,10,6,0,13,4,250
	L = 13,10,7,4,13,0,0,13,250
	M = 13,10,7,4,0,6,0,9,0,0,6,13,10,7,4,250
	N = 13,10,7,4,0,7,0,10,0,13,10,7,4,250
	O = 10,7,13,4,0,13,4,0,13,4,10,7,250
	P = 13,10,7,4,9,0,4,9,0,4,7,250
	Q = 10,7,13,4,13,4,9,13,4,10,7,13,250
	R = 13,10,7,4,9,0,4,9,4,11,7,13,250
	S = 12,6,8,13,4,8,13,4,9,11,5,250
	T = 4,0,4,13,10,7,4,0,4,0,4,250
	U = 10,7,4,13,0,0,13,0,0,13,10,7,4,250
	V = 10,7,4,12,0,13,0,12,0,0,10,7,4,250
	W = 11,8,4,13,0,11,8,0,13,0,0,10,7,4,250
	X = 13,4,11,6,0,8,0,10,6,13,4,250
	Y = 4,0,7,13,10,0,7,0,4,250
	Z = 13,0,4,11,13,4,9,13,4,7,13,4,250

	/ = 13,10,7,4,250
	- = 9,0,9,0,9,0,9,0,9,250
	+ = 9,0,9,12,9,6,9,0,9,250
	= = 10,7,10,7,10,7,10,7,10,7,250
	. = 13,13,250
	? = 6,13,13,4,10,0,4,9,0,6,250
	! = 13,13,10,7,4,250
	' = 7,4,250
	" = 7,4,0,7,4,250
	( = 9,6,12,0,4,13,250
	) = 13,4,0,12,6,9,250
	\ = 4,7,10,13,250
	: = 10,10,5,5,250
	, = 13,0,12,10,250
    _ = 13,13,13,13,13,13,13,13,13,250
	
	degree = 7,5,8,4,7,5,250
	check mark = 11,0,13,10,7,4,250

	triangle (large, point up) = 13,0,10,13,7,13,4,13,7,13,10,0,13,250
	triangle (large, point down) = 4,0,7,4,10,4,13,4,10,4,7,0,4,250
	triangle (small, point up) = 10,0,7,10,4,10,7,0,10,250
	triangle (small, point down) = 4,0,7,4,10,4,7,0,4,250
_______________________________________________________________________________________________________________________________
#endrem

#terminal 19200									'open PICAXE IDE serial terminal screen after program compiles

setfreq m16										'run PICAXE M2+ @ 16 MHz clock speed

symbol DDS_CTRL = $0							'$0 DDS bit load control for AD9850 while AD9851 uses $9 (change as required)

#define WSPR									'comment out if you don't want/need a WSPR bcn

'Define QRSS bcn operations by commenting out unwanted defines (only ONE mode used at a time)
'#define CW										'on-off carrier keying (OOK or OOC) CW
'#define DFCW									'dual freq shift keying (DFCW or DFSK) CW
'#define FSKCW									'freq shift keying (FSK} CW
'#define SMTCW									'sequential multitone (SMT) graphical CW
#define TRICW									'triangle wave CW

'SMT/SCR or QRSS bcns encoded text or standard Morse code encoded text values here (200 bytes reserved)
#ifdef SMTCW

	symbol DIT_TIME = 1							'QRSS 1 dit time length (tx on) in secs
	
	'"ZZ9ZZZ 23" (23 dBm/200 mW) QRSS STM graphical CW bcn text callsign & optionsl power in dBm (use SMT encoding table)
	'250 marks end of character add spacing; 251 marks end of group add word spacing; 255 marks end of text/bcn
	eeprom 0,(13,0,4,11,13,4,9,13,4,7,13,4,250,_
			  13,0,4,11,13,4,9,13,4,7,13,4,250,_
			  13,6,9,4,13,9,4,11,9,6,250,_
		  	  13,0,4,11,13,4,9,13,4,7,13,4,250,_
 		  	  13,0,4,11,13,4,9,13,4,7,13,4,250,_
 		  	  13,0,4,11,13,4,9,13,4,7,13,4,251,_
   		  	  13,0,11,13,4,9,13,4,13,7,250,_
		  	  13,0,13,4,8,13,4,8,11,6,255)

#else

	#ifdef TRICW								'for TRICW
		
		symbol DIT_TIME = 1						'QRSS 1 dit time length (tx on) in secs
		
		'"ZZ9ZZZ UV" (23 dBm/200 mW) triangle CW bcn text callsign & optional power in dBm (used Morse code encoding table)
		'use "cut" numbers for power in dBm (buy not allowed in callsigns)
		'0 marks end of group add word/group spacing, 255 marks end of text/bcn
		eeprom 0,(19,19,47,19,19,19,0,12,24,255)

	#else										'for all other QRSS CW modes
	
		symbol DIT_TIME = 6						'QRSS 6 dit time length (tx on) in secs

		'"ZZ9ZZZ" QRSS CW, FSKCW, DFCW bcn text callsign only (use Morse code encoding table)
		'0 adds word/group spacing, 255 marks end of bcn [text]
		eeprom 0,(19,19,47,19,19,19,255)

	#endif

#endif

#ifdef WSPR										'for WSPR include following code
	
	'WSPR CHARACTER/symbol transmission time (based on 16 MHz clock)
	'adjust as required +/- for clock ppm error (WSPR symbol transmission time 683 milliseconds for total of 110.6 secs)
	symbol WSPR_CLK	= 123						'WSPR clock value used in tx loop (adjust as required)
	symbol WSPR_DELAY = 683						'WSPR symbol tx time on (adjust as required)
	
	symbol WSPR_FRAME = 120						'WSPR tx frame in secs (to start QRSS bcn on next minute after WSPR bcn)
		
	symbol WSPR_SYMBOL = b0						'WSPR bcn LOOKUP table start address (255 bytes reserved)

	symbol WSPR_0 = 240							'WSPR symbol 0 freq EEPROM address
	symbol WSPR_1 = 244							'WSPR symbol 1 freq EEPROM address
	symbol WSPR_2 = 248							'WSPR symbol 2 freq EEPROM address	
	symbol WSPR_3 = 252							'WSPR symbol 3 freq EEPROM address

	'NB *** You must put 4-byte (hex values) freq in reverse order from values given by website calculator! ***
	'http://beta-tools.analog.com/adisimdds/#/?part=AD9850&fin=125&mult=1&ftw=14C4415B&rso=111111&harmonicDB=-50&useFilters=0

	'WSPR symbol frequencies use 1.46 Hz spacing from WSPR symbol 0 (carrier} freq (stored in EEPROM locations 0 to 161)
	'30 m WSPR frequencies where EEPROM address is: DDS_FREQ = symbol/character * 4 + WSPR_0 value
	'eeprom WSPR_0,($9C,$67,$C4,$14)				'10.140.275.00 MHz (WSPR symbol 0)
	'eeprom WSPR_1,($CE,$67,$C4,$14)				'10.140.276.46 MHz (WSPR symbol 1)
	'eeprom WSPR_2,($00,$68,$C4,$14)				'10.140.277.92 MHz (WSPR symbol 2)
	'eeprom WSPR_3,($32,$68,$C4,$14)				'10.140.279.38 MHz (WSPR symbol 3)

	'30 m WSPR frequencies where EEPROM address is: DDS_FREQ = symbol/character * 4 + WSPR_0 value
	eeprom WSPR_0,($F0,$10,$6B,$0E)				'7.040.150.00 MHz (WSPR symbol 0)
	eeprom WSPR_1,($22,$11,$6B,$0E)				'7.040.151.46 MHz (WSPR symbol 1)
	eeprom WSPR_2,($54,$11,$6B,$0E)				'7.040.152.92 MHz (WSPR symbol 2)
	eeprom WSPR_3,($87,$11,$6B,$0E)				'7.040.154.38 MHz (WSPR symbol 3)

#endif

'(Generated with 'WSPRMSG.exe' program by Gene Marcus, W3PM, (c) January 2013 (change as required)
'WSPR bcn symbols (162 bytes reserved): callsign, grid square & power (dBm)
#macro get_WSPR_symbol(CHARACTER)				'macro has to be defined outside of #ifdef WSPR/#endif block
		
	'"ZZ9ZZZ AA58 23" (23 dBm/200 mW)
	lookup WSPR_SYMBOL,(3,1,0,0,0,2,2,2,3,0,2,2,3,3,1,2,2,2,3,2,0,1,2,1,1,1,1,0,2,2,2,2,_
						0,0,3,0,2,1,2,3,2,2,0,2,2,0,3,0,1,1,0,0,3,3,2,1,2,0,0,3,3,2,3,2,_
						2,2,0,1,3,2,3,2,1,2,1,0,1,0,0,3,2,2,1,2,1,3,2,2,0,3,1,2,1,2,3,0,_
						2,0,3,2,0,2,0,2,3,2,2,1,2,0,1,3,3,0,3,3,2,2,3,1,2,1,2,0,0,1,1,3,_
						2,2,2,2,2,3,0,1,2,0,3,3,0,0,2,0,2,2,0,1,1,0,1,2,3,3,0,0,2,1,3,0,2,0),CHARACTER
								
#endmacro

'QRSS frequencies 4 hex byte x 10 stored in EEPROM (starts after WSPR bcn data
symbol QRSS_0 = 200							'freq 0 EEPROM address
symbol QRSS_1 = 204							'freq 1 EEPROM address
symbol QRSS_2 = 208							'freq 2 EEPROM address
symbol QRSS_3 = 212							'freq 3 EEPROM address
symbol QRSS_4 = 216 						'freq 4 EEPROM address
symbol QRSS_5 = 220							'freq 5 EEPROM address
symbol QRSS_6 = 224							'freq 6 EEPROM address
symbol QRSS_7 = 228							'freq 7 EEPROM address
symbol QRSS_8 = 232							'freq 8 EEPROM address
symbol QRSS_9 = 236							'freq 9 EEPROM address

'for all QRSS modes can select any carrier/space & mark/shift freq pair (5 Hz space/mark used here)
symbol SPAC_FREQ = QRSS_0					'space freq EEPROM address
symbol MARK_FREQ = QRSS_4					'mark freq EEPROM address	

'NB *** You must put 4-byte (hex values) freq in reverse order from values given by website calculator! ***
'http://beta-tools.analog.com/adisimdds/#/?part=AD9850&fin=125&mult=1&ftw=14C4415B&rso=111111&harmonicDB=-50&useFilters=0
	
'30 m frequencies (ten SMT graphical CW frequencies with 1 Hz spacing (also used for triangle CW)
'eeprom QRSS_0,($57,$3A,$C4,$14)				'10.139.900 MHz
'eeprom QRSS_1,($79,$3A,$C4,$14)				'10.139.901 MHz
'eeprom QRSS_2,($9C,$3A,$C4,$14)				'10.139.902 MHz
'eeprom QRSS_3,($BE,$3A,$C4,$14)				'10.139.903 MHz
'eeprom QRSS_4,($E1,$3A,$C4,$14)				'10.139.904 MHz
'eeprom QRSS_5,($03,$3B,$C4,$14)				'10.139.905 MHz
'eeprom QRSS_6,($25,$3B,$C4,$14)				'10.139.906 MHz
'eeprom QRSS_7,($48,$3B,$C4,$14)				'10.139.907 MHz
'eeprom QRSS_8,($6A,$3B,$C4,$14)				'10.139.908 MHz
'eeprom QRSS_9,($8C,$3B,$C4,$14)				'10.139.909 MHz

'40 m frequencies (ten SMT graphical CW frequencies with 1 Hz spacing (also used for triangle CW)
eeprom QRSS_0,($F6,$E1,$6A,$0E)				'7.039.800 MHz
eeprom QRSS_1,($19,$E2,$6A,$0E)				'7.039.801 MHz
eeprom QRSS_2,($3B,$E2,$6A,$0E)				'7.039.802 MHz
eeprom QRSS_3,($5B,$E2,$6A,$0E)				'7.039.803 MHz
eeprom QRSS_4,($80,$E2,$6A,$0E)				'7.039.804 MHz
eeprom QRSS_5,($A2,$E2,$6A,$0E)				'7.039.805 MHz
eeprom QRSS_6,($C4,$E2,$6A,$0E)				'7.039.806 MHz
eeprom QRSS_7,($E7,$E2,$6A,$0E)				'7.039.807 MHz
eeprom QRSS_8,($09,$E3,$6A,$0E)				'7.039.808 MHz
eeprom QRSS_9,($2B,$E3,$6A,$0E)				'7.039.809 MHz

'end user_setup
'_______________________________________________________________________________________________________________________________

'program_setup

'Pause_Time subroutine (so we can use DS3231 RTC 1-PPS very accurate timing & replace PICAXE's internal timer)
#macro Pause_Time(PAUSE_LENGTH)

	OLD = PPS									'save current RTC time value (in secs)
	
	do
	
		NEW = PPS - OLD							'calculate NEW elapsed time (in secs)
		
	loop while NEW < PAUSE_LENGTH				'loop until equal to required PAUSE_LENGTH (in secs)

#endmacro

'_______________________________________________________________________________________________________________________________

'AD9850/AD9851 control commands Paul Wilton, M1CNK, (c) 2009
'Slightly modified by Robert C. Mazur, VA3ROM, July 2016 for PICAXE 08M2+

'FQ_UD EEPROM freq byte data into AD9850/AD9851 & set new freq & turn on RF output
'Hex bytes defining freq are stored in reverse order (makes coding way easier!)
#macro DDS_Tune(DDS_FREQ)						'macro lets us pass compound equation in one line instead of two (as with subs)

	low FQ_UD									'set DDS LOAD line low for freq programming
	
	for COUNTER2 = 0 to 3						'4-bytes (32-bits) total to load
		
		COUNTER3 = DDS_FREQ + COUNTER2			'point to EEPROM memory area with our freq hex data
		
		read COUNTER3, HEX_BYTE					'read next hex byte from EEPROM

		gosub Shiftout_LSBFirst					'load FQ_UD register with hex data (LSB first)

	next COUNTER2
	
	HEX_BYTE = DDS_CTRL							'AD9850 uses $0 while AD9851 uses $9
	
	gosub Shiftout_LSBFirst						'send hex byte control command to DDS (LSB first)

	pulsout FQ_UD, 1 							'finished loading freq & control command so generate RF signal

#endmacro
'_______________________________________________________________________________________________________________________________

symbol TRANSMIT_FRAME = 600						'WSPR + QRSS transmission time frame 600 secs (10 mins)

symbol CW_SPC = 250								'SMT CW end of character single add character spacing (not Morse routine)
symbol CW_WRD = 251								'SMT CW or Morse routine end of character group (word) add word spacing
symbol CW_EOT = 255								'SMT CW or Morse routine end of character text/bcn stop transmission

symbol CHARACTER = b1							'WSPR & QRSS CW character byte variable (used by WSPR & Morse code subroutines)
symbol INDEX = b2								'current EEPROM memory location index (pointer)

symbol MASK = b3 								'DDS bit masking variable
symbol HEX_BYTE = b4 							'DDS data variable

symbol COUNTER1 = b5 							'DDS tune/loop variable
symbol COUNTER2 = b6							'DDS tune/loop variable
symbol COUNTER3 = b7							'DDS tune/loop variable

symbol DDS_FREQ = b8	 						'sets EEPROM freq address to use

symbol PAUSE_LENGTH = b9						'various timings variable (replaces PICAXE pause function)'

symbol J = b10									'general purpose use byte variable
symbol K = b11									'general purpose use byte variable

symbol X = W9									'general purpose use math word variable

symbol TICKS = w10								'general purpose use word seconds "tick" variable

symbol PPS = w11								'counts number of DS3231 1-pulse-per-second pulses (PPS)

symbol NEW = w12								'used for time difference maths (replaces PAUSE function)
symbol OLD = w13								'used for time difference maths (replaces PAUSE function)

output C.0, C.1, C.2							'set output pins (by default they are input)

'AD9850/AD9851 connections to PICAXE (AD9850/AD9851 RESET tied to circuit ground)
symbol S_DATA = C.0								'S_DATA (data) connected C.0 (pin 7)
symbol FQ_UD = C.1								'FQ_UD input connected to C.1 (pin 6, SCL)
symbol W_CLK = C.2								'W_CLK (clock) connected to C.2 (pin 5, SDA)

symbol PUSH_BUTTON = pinC.3						'push button circuit connected to C.3 (pin 4)
symbol DS3231_SQW = pinC.4						'DS3231 SQW (square wave) 1-PPS connected to C.4 (pin 3)

'define various QRSS beacons dot, space & word time lengths (change as required)
#ifdef CW										'for QRSS CW normal Morse code timings only
		
	symbol DIT_FREQ = SPAC_FREQ					'EEPROM dit freq address (carrier/space only no mark/shift)
	symbol DAH_FREQ = SPAC_FREQ					'EEPROM dah freq address (carrier/space only no mark/shift)

	symbol DAH_TIME = DIT_TIME * 3				'for standard Morse code
	
	'symbol SPAC_TIME = DIT_TIME * 3			'standard Morse code character spacing time length (tx off) in secs	
	symbol SPAC_TIME = DIT_TIME * 2				'for 2/3 Morse code character spacing

#endif

#ifdef DFCW										'for DFCW only
		
	symbol DIT_FREQ = SPAC_FREQ					'EEPROM dit freq address (carrier/space)
	symbol DAH_FREQ = MARK_FREQ					'EEPROM dah freq address (mark/shift)	

	symbol DAH_TIME = DIT_TIME					'dit & dah have same Morse code timings

	'symbol SPAC_TIME = DIT_TIME * 3			'standard Morse code character spacing
	symbol SPAC_TIME = DIT_TIME * 2				'for 2/3 Morse code character spacing

#endif

#ifdef FSKCW									'for FSKCW normal Morse code timings only

	symbol DIT_FREQ = MARK_FREQ					'EEPROM dit freq address (mark/shift)  
	symbol DAH_FREQ = MARK_FREQ					'EEPROM dah freq address (mark/shift)

	symbol DAH_TIME = DIT_TIME * 3				'for standard Morse code

	'symbol SPAC_TIME = DIT_TIME * 3			'standard Morse code character spacing
	symbol SPAC_TIME = DIT_TIME * 2				'for 2/3 Morse code character spacing
	
#endif

#ifdef SMTCW									'for SMTCW only
	
	symbol DIT_FREQ = QRSS_0					'EEPROM dit freq address (starts at 0 & increments up to 9 (as required)
	symbol DAH_FREQ = QRSS_0					'defined but not used
	
	symbol DAH_TIME = DIT_TIME					'defined but not used
	
	symbol SPAC_TIME = DIT_TIME * 3				'character spacing time length (tx off) in secs						
	symbol WORD_TIME = SPAC_TIME * 2			'word/group spacing time length (tx off) in secs
	
#endif

#ifdef TRICW									'for TRICW only

	symbol DIT_FREQ = QRSS_4					'EEPROM dit freq address (starts at 0 & increments up to 9 (as required)
	symbol DAH_FREQ = QRSS_4					'defined but not used
	
	symbol DAH_TIME = DIT_TIME					'defined but not used
	
	symbol SPAC_TIME = DIT_TIME * 5					
	
#endif

'end user_setup
'_______________________________________________________________________________________________________________________________

'initialize_program

'enable I2C communications with DS3231 RTC module (100 kbps) @ PICAXE default clock speed (4 MHz)
'connect to DS3231 RTC module using SCL/SDA (physical PICAXE 08M2 pins 6 & 5) also shared with AD9850/AD9851
hi2csetup i2cmaster, %11010000, i2cslow, i2cbyte

'square wave outputs: $0 = 1 Hz, $8 = 1024 Hz, $10 = 4096 Hz & $18 = 8192 Hz, $4 = off
'enable 1-PPS square wave output (SQW) required at each power up
hi2cout $0E, ($0)

'AD9850/AD9851 shares same lines (SCL/SDA) with DS3231 RTC so we need to turn I2C off
hi2csetup off

gosub DDS_Init									'initialize module (required on each power on cycle)
	gosub DDS_Off								'turn off DDS RF outputs (square & sine)

'set [polling] interrupt for rising pulse (high) from DS3231 SQW 1-PPS output (replaces PICAXE internal timer)
setint %00010000, %00010000					   	'set polling interrupt for C.4 (pin 3) connected to DS3231 SQW output

sertxd("Press tx btn", cr, lf)

'routine to start WSPR bcn at exact even minute using push button connected to C.3 (pin 4)
do loop until PUSH_BUTTON = 1					'wait for push button press (high or 1) before starting program

'end program initialization
'_______________________________________________________________________________________________________________________________

'main_program
Main:
	
	PPS = 0										'reset SQW pulse-per-second count from DS3231 RTC module
	
	#ifdef WSPR									'if WSPR bcn active
		
		gosub WSPR_Transmit						'transmit WSPR bcn first
	
	#endif 


	#ifdef SMTCW								'for SMTCW only
	
		gosub SMTCW_Transmit
	
	#else										'for all other QRSS modes
		
		gosub QRSS_Transmit						'transmit QRSS CW, DFCW, FSKCW or triangle CW bcn
		
	#endif 

	sertxd("Wait", cr, lf)
	
	'10 mins transmit frame used so WSPR + QRSS or QRSS only bcn must not exceed this time
	do loop while PPS < TRANSMIT_FRAME
	
goto Main										'repeat bcn sequence forever or until power removed

end												'end of main program (prevents crashing into subroutines if something goes awry)

'Interrupt subroutine (comes here when DS3231 1-PPS detected)
Interrupt:
	
	if DS3231_SQW = 1 then Interrupt			'wait for SQW 1-PPS signal from DS3231 RTC to go low (0)
	
	inc PPS										'increment PPS counter 
	
	setint %00010000, %00010000				   	'reset polling interrupt (always required for PICAXE)
	
return

'WSPR bcn transmit subroutine
#ifdef WSPR										'if WSPR defined included this subroutine
	
	WSPR_Transmit:
	
		TICKS = PPS									'save current PPS counter value
			
		sertxd("Tx WSPR bcn", cr, lf)
	
		'transmit WSPR bcn stored in EEPROM (162 characters/symbols * 683 ms = 110.6 secs total time)
		for WSPR_SYMBOL = 0 to 161
		
			get_WSPR_symbol(CHARACTER)			'get WSPR symbol -- returned in CHARACTER variable
			
			'calculate EEPROM address pointer for character/symbol freq & tune DDS
			DDS_Tune(CHARACTER * 4 + WSPR_0)
		
			'transmit WSPR character/symbol for approx 683 ms (adjust WSPR_DELAY as required)
			for J = 0 to WSPR_CLK
			
				pauseus WSPR_DELAY
				
			next J
		
	next WSPR_SYMBOL							'repeat till all WSPR bcn symbols transmitted
		
	gosub DDS_Off
			
		TICKS = PPS - TICKS						'calculate beacon transmission time & display
		
		sertxd("WSPR bcn secs ") sertxd(#TICKS, cr, lf)
	
		'idle here till start of next minute before transmitting QRSS bcn (if defined)
		do loop while PPS < WSPR_FRAME
	
	return 
	
#endif
'_______________________________________________________________________________________________________________________________

'QRSS CW, DFCW, or FSKCW transmit subroutine
#ifndef SMTCW									'SMTCW not defined include this subroutine
	
	QRSS_Transmit:

		TICKS = PPS								'save current PPS counter value

		sertxd("Tx QRSS bcn", cr, lf)
		
		#ifdef FSKCW							'for FSKCW only (transmit carrier lead-in)
   		
			DDS_Tune(SPAC_FREQ)
		
  			Pause_Time(SPAC_TIME)
		
		#endif
 	
		INDEX = 0								'reset index pointer
		
		'Morse code routine follows for QRSS CW FSK CW & DFCW
		do										'read characters one at a time
			
			read INDEX, CHARACTER				'get Morse code character to transmit from EEPROM memory
		
			if CHARACTER = CW_EOT then exit		'end of transmission (bcn) marker byte all done exit do/loop
			
			gosub Morse_Transmit				'else transmit character using Morse code
   			 
			inc INDEX							'increment pointer
			
			#ifndef FSKCW						'FSKCW not defined
					
				'following prevents tx of extra character space at end of bcns (affects actual bcn tx time & sequencing)
				read INDEX, CHARACTER 			'take a peek at next character in EEPROM memory
			
				if CHARACTER <> CW_EOT then		'more characters to transmit?
				
					gosub Char_Space			'add character spacing if more characters to transmit
            
				endif
			
			#else
			
				gosub Char_Space				'always add spacing for FSKCW (also for end of bcn carrier lead-out)
				
			#endif
		
		loop									'go back for another character
	
		gosub DDS_Off
	
		TICKS = PPS - TICKS						'calculate beacon transmission time & display
		
		sertxd("QRSS bcn secs ") sertxd(#TICKS, cr, lf)
	
	return

#endif 
'_______________________________________________________________________________________________________________________________

'SMTCW subroutine transmits sequential multitone graphical CW
#ifdef SMTCW

	SMTCW_Transmit:

		TICKS = PPS								'save current PPS counter value

		sertxd("Tx SMT bcn", cr, lf)
		
		INDEX = 0								'reset index pointer
		
		do
		
			read INDEX, CHARACTER				'get Morse code character to transmit from EEPROM memory
					
			select case CHARACTER
			
				case CW_SPC						'character space marker
			
					Pause_Time(SPAC_TIME)
				
				case CW_WRD						'word/group space marker
			
					Pause_Time(WORD_TIME)					
					
				case CW_EOT						'end of text marker
				
					exit
			
				else
			
					'calculate SMT CW freq start (converted from MEPTController file format) & tune DDS
					DDS_Tune(13 - CHARACTER * 4 + QRSS_0)
			
					Pause_Time(DIT_TIME)
					
			endselect	
		
			inc INDEX
		
			gosub DDS_Off
			
		loop
	
		TICKS = PPS - TICKS						'calculate beacon transmission time & display
		
		sertxd("SMT bcn secs ") sertxd(#TICKS, cr, lf)

	return

#endif
'_______________________________________________________________________________________________________________________________

'Triangle CW transmit subroutine (global variable "J" contains value generated by Morse_Transmit)
#ifdef TRICW

	TRICW_Transmit:
	
		for K = 0 to 7
			
			'triangle wave generation lookup table
			lookup K,(0,4,8,12,16,12,8,4),X
			
			'calculate DDS frequency to generate (triangle CW is a hybrid mix of DFCW/FSKCW, ramp & slant CW)
			if J = 1 then
					
				'triangle CW dah rises 5 Hz above carrier & back down
				DDS_Tune(DIT_FREQ + X)
			
			else
				
				'triangle CW dit drops 5 Hz below carrier & back up
				DDS_Tune(DIT_FREQ - X)
				
			endif
			
			'calculate SMT CW freq EEPROM memory address (convert from MEPTController file format) & tune DDS
				
			Pause_Time(DIT_TIME)
			
		next K
	
	return
					
#endif
'_______________________________________________________________________________________________________________________________

'Routine to transmit Morse code characters using bit shifting 
'Based on original code by Steve Paige, VK6HV                  
'Modified for QRSS by Robert C. Mazur, VA3ROM (July 2016)     
#ifndef SMTCW									'SMTCW not active add Morse code QRSS CW routines

	Morse_Transmit:  			
		
		'check for word/group space first
		if CHARACTER = 0 then
		
			return
	
		end if			

		'right bit shifting (division) routine will keep going until Morse code character shifted value reaches 1
		do

			J = CHARACTER & 1					'mask bit0 (odd numbers return 1, even numbers return 0)
			
			if J = 1 then						'if result is odd 1 (dah/dash) else even 0 (dit/dot)
				
				#ifndef TRICW					'for all QRSS CW modes except TRICW
				
					gosub Dah					'send dah (dash) character element
 	      
				#else
				
		  			gosub TRICW_Transmit
					
				#endif
			
	   		else
	     		
				#ifndef TRICW
					
					gosub Dit					'send dit (dot) character element
	
				#else
				
					gosub TRICW_Transmit
					
				#endif
			
			endif
		
			CHARACTER = CHARACTER / 2      		'division by 2 performs binary right bit shift
  			
			#ifndef TRICW						'no character element spacing used by TRICW
				
				'add required character element spacing 
 				if CHARACTER <> 1 then				
				
					gosub Gap_Space				'wait 1 dit length for character element spacing
				
				endif
	
			#endif
		
  	loop until CHARACTER = 1					'loop until all bits shifted & value 1 is reached
					  		  
return

'Dit subroutine
	Dit:

		DDS_Tune(DIT_FREQ)
	
  		Pause_Time(DIT_TIME)
		
	return

'Dah subroutine
	Dah:
	  
		DDS_Tune(DAH_FREQ)
  
		Pause_Time(DAH_TIME)
  
	return

'Gap spacing subroutine
	Gap_Space:             						'space between character elements

		#ifdef FSKCW							'for FSKCW only
 
			DDS_Tune(SPAC_FREQ)

		#else									'for all other QRSS modes
		
			gosub DDS_Off
			
		#endif	
	
		Pause_Time(DIT_TIME)
		
	return

#endif
'_______________________________________________________________________________________________________________________________

'Character spacing subroutine
	Char_Space:									'space between characters

		#ifdef FSKCW							'for FSKCW only
  	
			DDS_Tune(SPAC_FREQ)
  													
		#else									'for all other QRSS modes
		
			gosub DDS_Off
		
		#endif
		
		Pause_Time(SPAC_TIME)					'keep transmitter turned off for required time
	
	return

'AD9850/AD9851 control commands Paul Wilton, M1CNK, (c) 2009
'Slightly modified by Robert C. Mazur, VA3ROM, July 2016 for PICAXE 08M2+

'Initialize AD9850/AD9851 DDS module (always required at start up)
DDS_Init:

	'prepare DDS for initialization (pull all registers low or 0) DDS RESET (RST) tied to circuit ground (so always low or 0)
	low FQ_UD
		
		low S_DATA
	
	low W_CLK
	
	pause 2										'wait a bit
	
	'strobe CLOCK to get hardware bits D0, D1 & D2 into input register & push LOAD high (1) to get them into control register
	high W_CLK
	
		low W_CLK
	
	high FQ_UD
	
	pause 2										'wait a bit
	
return

'AD9850/AD9851 40-bit freq/control register loading (LSB first, 4 hex bytes (32-bits) for freq
'One hex byte (8-bits) used for command/control & signal phase
Shiftout_LSBFirst:

	for COUNTER1 = 0 to 7 						'one hex byte is 8 bits

		MASK = HEX_BYTE & 1 					'mask bit0 to see if it's set									

		low S_DATA 								'set DATA line low to accept input

		if MASK <> 0 then						'is bit0 set (1)?
		
			high S_DATA							'yes, push DATA line high (1) for FQ_UD bit load

		endif

		pulsout W_CLK, 1	 					'pulse CLOCK (10us) to shift to next bit register

		HEX_BYTE = HEX_BYTE / 2 				'right bit shift value
		
	next COUNTER1

return

'DDS Off subroutine (also clears previous DDS freq)
DDS_Off:

	HEX_BYTE = $2								'AD9850/AD9851 "off" command code
	
	low FQ_UD									'pull DATA line low (0)
	
	gosub Shiftout_LSBFirst						'load FQ_UD register with DDS "off" hex byte (LSB first)

	high FQ_UD									'push DATA line high (1) & execute command
	
return

